local AceGUI = LibStub("AceGUI-3.0")

local btnStart, btnLastCall, editCount, btnRolling, btnJoin, btnRoll, btnReset;

local phase; --0 - off, 1 - Inviting, 2 - Last Call, 3 - Rolling
local count = 50;
local rollers = {};
local low = {};
local high = {};
local minimum = 0;
local maximum = count;
local countInvited = 0;
local countRolled = 0;
local countInvitedR = 0;
local countRolledR = 0;
local localGambleStatistics = {};
local trades = {};
local tradesNew = {};
local inviteTime = 10;
local lastCallTime = 3;
local notRolledTimeInterval = 10;
local activeTimerID = nil;
local rollType = 0;

function MorthalinRaidTools:ConstructGamble()
	self.ui.gambleFrame = AceGUI:Create("Frame");
	self.ui.gambleFrame:SetTitle("Gamble");
	self.ui.gambleFrame:SetStatusText("");
	self.ui.gambleFrame:SetLayout("Flow");
	self.ui.gambleFrame:SetWidth(345);
	self.ui.gambleFrame:SetHeight(230);
	self.ui.gambleFrame:EnableResize(false);
	self.ui.gambleFrame.frame:EnableMouse(true);
	self.ui.gambleFrame.frame:SetUserPlaced(true);
	phase = 0;
	
	btnStart = AceGUI:Create("Button");
	btnStart:SetText("Start gamble");
	btnStart:SetCallback("OnClick", function() self:StartGamble() end)
	self.ui.gambleFrame:AddChild(btnStart);
	
	editCount = AceGUI:Create("EditBox");
	editCount:SetLabel("Roll pool:");
	editCount:SetWidth(100);
	editCount:SetText(tostring(count));
	editCount:DisableButton(true);
	editCount:SetCallback("OnTextChanged", function(widget, event, text)
		if(phase == 0) then
			count = text;
			if(string.len(text) == 0) then
				btnStart:SetDisabled(true);
			else
				btnStart:SetDisabled(false);
			end
		end
	end)
	self.ui.gambleFrame:AddChild(editCount);
	
	btnLastCall = AceGUI:Create("Button");
	btnLastCall:SetText("Last Call");
	btnLastCall:SetCallback("OnClick", function() if(activeTimerID ~= nil) then MorthalinRaidTools:CancelTimer(activeTimerID); end self:LastCall(); end)
	btnLastCall:SetDisabled(true);
	self.ui.gambleFrame:AddChild(btnLastCall);
	
	btnJoin = AceGUI:Create("Button");
	btnJoin:SetText("Join Game");
	btnJoin:SetWidth(100);
	btnJoin:SetCallback("OnClick", function() SendChatMessage("1", "RAID", "Common"); btnJoin:SetDisabled(true); end)
	btnJoin:SetDisabled(true);
	self.ui.gambleFrame:AddChild(btnJoin);
	
	btnRolling = AceGUI:Create("Button");
	btnRolling:SetText("Start rolling");
	btnRolling:SetCallback("OnClick", function() if(activeTimerID ~= nil) then MorthalinRaidTools:CancelTimer(activeTimerID); end self:StartRolls(); end)
	btnRolling:SetDisabled(true);
	self.ui.gambleFrame:AddChild(btnRolling);
	
	btnRoll = AceGUI:Create("Button");
	btnRoll:SetText("Roll");
	btnRoll:SetWidth(100);
	btnRoll:SetCallback("OnClick", function() RandomRoll(1, count); btnRoll:SetDisabled(true); end)
	btnRoll:SetDisabled(true);
	self.ui.gambleFrame:AddChild(btnRoll);
	
	btnReset = AceGUI:Create("Button");
	btnReset:SetText("Reset");
	btnReset:SetCallback("OnClick", function() if(activeTimerID ~= nil) then MorthalinRaidTools:CancelTimer(activeTimerID); end self:ResetGamble(); end)
	btnReset:SetDisabled(true);
	self.ui.gambleFrame:AddChild(btnReset);
	
	local btn = AceGUI:Create("Button");
	btn:SetText("Trades");
	btn:SetWidth(100);
	btn:SetCallback("OnClick", function() self:ShowTrades(); end)
	self.ui.gambleFrame:AddChild(btn);
	
	btn = AceGUI:Create("Button");
	btn:SetText("Global statistics");
	btn:SetCallback("OnClick", function() self:GetGlobalStatistics(); end)
	self.ui.gambleFrame:AddChild(btn);
	
	btn = AceGUI:Create("Button");
	btn:SetText("Actual stat");
	btn:SetWidth(100);
	btn:SetCallback("OnClick", function() self:GetLocalStatistics(); end)
	self.ui.gambleFrame:AddChild(btn);
	
	self.ui.gambleFrame:Hide();
end

function MorthalinRaidTools:ShowGamble()
	if(self.ui.gambleFrame:IsVisible() == false) then
		self.ui.gambleFrame:Show();
	end
end

function MorthalinRaidTools:HideGamble()
	if(self.ui.gambleFrame:IsVisible()) then
		self.ui.gambleFrame:Hide();
	end
end

function MorthalinRaidTools:AcceptInvGamble()
	return phase == 1 or phase == 2;
end

function MorthalinRaidTools:AcceptRollsGamble()
	return phase == 3;
end

function MorthalinRaidTools:StartGamble()
	activeTimerID = nil;
	phase = 1;
	self.ui.gambleFrame:SetStatusText("Inviting players");
	SendChatMessage("---Try your luck and join Morthalin Gamble. Write 1 to join. Rolling " .. count .. " ---", "RAID", "Common");
	btnStart:SetDisabled(true);
	btnLastCall:SetDisabled(false);
	btnLastCall:SetCallback("OnEnter", function() MorthalinRaidTools:ShowInvitedGamble(); end);
	btnLastCall:SetCallback("OnLeave", function() MorthalinRaidTools:HideTooltip(); end);
	editCount:SetDisabled(true);
	btnReset:SetDisabled(false);
	btnJoin:SetDisabled(false);
	if(MorthalinRaidTools.db.global.gambleAuto) then
		activeTimerID = MorthalinRaidTools:ScheduleTimer(function() MorthalinRaidTools:LastCall(); end, MorthalinRaidTools.db.global.gambleInviteTime);
	end
	if(MorthalinRaidTools.db.global.gambleJoin) then
		SendChatMessage("1", "RAID", "Common");
	end
	if(self.debug) then
		countInvited = countInvited + 1;
		rollers["tester1"] = 0;
		countInvited = countInvited + 1;
		rollers["tester2"] = 0;
		countInvited = countInvited + 1;
		rollers["tester3"] = 0;
	end
end

function MorthalinRaidTools:LastCall()
	activeTimerID = nil;
	phase = 2;
	SendChatMessage("---Last chance to join roll round. Write 1 to join.---", "RAID", "Common");
	btnLastCall:SetDisabled(true);
	btnRolling:SetDisabled(false);
	btnLastCall:SetCallback("OnEnter", nil);
	btnLastCall:SetCallback("OnLeave", nil);
	btnRolling:SetCallback("OnEnter", function() MorthalinRaidTools:ShowInvitedGamble(); end);
	btnRolling:SetCallback("OnLeave", function() MorthalinRaidTools:HideTooltip(); end);
	if(MorthalinRaidTools.db.global.gambleAuto) then
		activeTimerID = MorthalinRaidTools:ScheduleTimer(function() MorthalinRaidTools:StartRolls(); end, MorthalinRaidTools.db.global.gambleLastCallTime);
	end
end

function MorthalinRaidTools:ResetGamble()
	activeTimerID = nil;
	phase = 0;
	self.ui.gambleFrame:SetStatusText("");
	rollers = {};
	low = {};
	high = {};
	countInvited = 0;
	countRolled = 0;
	rollType = 0;
	btnStart:SetDisabled(false);
	btnLastCall:SetDisabled(true);
	editCount:SetDisabled(false);
	btnReset:SetDisabled(true);
	btnRolling:SetDisabled(true);
	btnJoin:SetDisabled(true);
	btnRoll:SetDisabled(true);
	btnLastCall:SetCallback("OnEnter", nil);
	btnLastCall:SetCallback("OnLeave", nil);
	btnRolling:SetCallback("OnEnter", nil);
	btnRolling:SetCallback("OnLeave", nil);
end

function MorthalinRaidTools:AddPlayerGamble(player)
	if(rollers[player] == nil) then
		countInvited = countInvited + 1;
		rollers[player] = 0;
	end
end

function MorthalinRaidTools:ShowInvitedGamble()
	GameTooltip:SetOwner(self.ui.gambleFrame.frame,"ANCHOR_RIGHT");
	GameTooltip:AddLine("Players");
	for name, roll in pairs(rollers) do
		GameTooltip:AddLine(name);
	end
	GameTooltip:Show();
end

function MorthalinRaidTools:StartRolls()
	if(countInvited < 2) then
		MorthalinRaidTools:ResetGamble();
	else
		phase = 3;
		self.ui.gambleFrame:SetStatusText("Rolls started");
		SendChatMessage("---Write /roll " .. count .. ". GL & HF.---", "RAID", "Common");
		btnRolling:SetDisabled(true);
		rollType = 1;
		btnReset:SetCallback("OnEnter", function() MorthalinRaidTools:ShowNoRolled(); end);
		btnReset:SetCallback("OnLeave", function() MorthalinRaidTools:HideTooltip(); end);
		
		btnJoin:SetDisabled(true);
		btnRoll:SetDisabled(false);
		if(MorthalinRaidTools.db.global.gambleAuto) then
			activeTimerID = MorthalinRaidTools:ScheduleRepeatingTimer(function() MorthalinRaidTools:CallNoRolled(); end, MorthalinRaidTools.db.global.gambleRollReminder);
		end
		
		if(self.debug) then
			rollers["tester1"] = tonumber(math.random(1,count));
			countRolled = countRolled + 1;
			rollers["tester2"] = tonumber(math.random(1,count));
			countRolled = countRolled + 1;
			rollers["tester3"] = tonumber(math.random(1,count));
			countRolled = countRolled + 1;
		end
		if(MorthalinRaidTools.db.global.gambleJoin) then
			MorthalinRaidTools:ScheduleTimer(function() RandomRoll(1, count); end, 1);
		end
	end
end

function MorthalinRaidTools:ShowNoRolled()
	GameTooltip:SetOwner(self.ui.gambleFrame.frame,"ANCHOR_RIGHT");
	GameTooltip:AddLine("Players");
	if(rollType == 1) then
		for name, roll in pairs(rollers) do
			if(roll == 0) then
				GameTooltip:AddLine(name);
			end
		end
	elseif(rollType == 2) then
		for name, roll in pairs(low) do
			if(roll == 0) then
				GameTooltip:AddLine(name);
			end
		end
	elseif(rollType == 3) then
		for name, roll in pairs(high) do
			if(roll == 0) then
				GameTooltip:AddLine(name);
			end
		end
	end
	GameTooltip:Show();
end

function MorthalinRaidTools:CallNoRolled()
	SendChatMessage("Players not rolled:", "RAID", "Common");
	if(rollType == 1) then
		for name, roll in pairs(rollers) do
			if(roll == 0) then
				SendChatMessage(name, "RAID", "Common");
			end
		end
	elseif(rollType == 2) then
		for name, roll in pairs(low) do
			if(roll == 0) then
				SendChatMessage(name, "RAID", "Common");
			end
		end
	elseif(rollType == 3) then
		for name, roll in pairs(high) do
			if(roll == 0) then
				SendChatMessage(name, "RAID", "Common");
			end
		end
	end
end

function MorthalinRaidTools:AllRolled()
	rollType = 0;
	local rollersSorted = {};
	low = {};
	high = {};
	
	for name, roll in pairs(rollers) do
		table.insert(rollersSorted , {name = name, roll = roll});
	end
	local function sortRollers(a, b)
		if(a.roll == b.roll) then
			return a.name < b.name;
		else
			return a.roll < b.roll;
		end
	end
	table.sort(rollersSorted, sortRollers);
	
	minimum = rollersSorted[1].roll;
	maximum = rollersSorted[#rollersSorted].roll;
	
	local minCount = 0;
	local index = 1;
	local looping = true;
	while looping do
		if(index > #rollersSorted or rollersSorted[index].roll ~= minimum) then
			looping = false;
		else
			low[rollersSorted[index].name] = 0;
			index = index + 1;
			minCount = minCount + 1;
		end
	end
	
	local maxCount = 0;
	index = #rollersSorted;
	looping = true;
	while looping do
		if(index < 1 or rollersSorted[index].roll ~= maximum) then
			looping = false;
		else
			high[rollersSorted[index].name] = 0;
			index = index - 1;
			maxCount = maxCount + 1;
		end
	end
	
	if(minCount > 1) then
		MorthalinRaidTools:MoreLow();
	elseif(maxCount > 1) then
		MorthalinRaidTools:MoreHigh();
	else
		MorthalinRaidTools:GetWinnerLooser();
	end
end

function MorthalinRaidTools:GetWinnerLooser()
	if(activeTimerID ~= nil) then 
		MorthalinRaidTools:CancelTimer(activeTimerID); 
	end
	local looser;
	local winner;
	local win = maximum - minimum;
	
	for name, roll in pairs(low) do
		looser = name;
	end
	for name, roll in pairs(high) do
		winner = name;
	end
	
	if(self.debug == false) then
		if(MorthalinRaidTools.db.global.gambleStatistics[looser] == nil) then
			MorthalinRaidTools.db.global.gambleStatistics[looser] = (-1 * win);
		else
			MorthalinRaidTools.db.global.gambleStatistics[looser] = MorthalinRaidTools.db.global.gambleStatistics[looser] - win;
		end
		
		if(MorthalinRaidTools.db.global.gambleStatistics[winner] == nil) then
			MorthalinRaidTools.db.global.gambleStatistics[winner] = win;
		else
			MorthalinRaidTools.db.global.gambleStatistics[winner] = MorthalinRaidTools.db.global.gambleStatistics[winner] + win;
		end
		
		if(localGambleStatistics[looser] == nil) then
			localGambleStatistics[looser] = (-1 * win);
		else
			localGambleStatistics[looser] = localGambleStatistics[looser] - win;
		end
		
		if(localGambleStatistics[winner] == nil) then
			localGambleStatistics[winner] = win;
		else
			localGambleStatistics[winner] = localGambleStatistics[winner] + win;
		end
	end
	
	SendChatMessage(winner .. " wins " .. tostring(win) .. "g from " .. looser .. ".", "RAID", "Common");
	table.insert(trades, looser .. " -> " .. winner .. ": " .. tostring(win) .. "g");
	
	if(tradesNew[looser] ~= nil and tradesNew[looser].winners[winner] ~= nil) then
		tradesNew[looser].winners[winner] = tradesNew[looser].winners[winner] + win;
	elseif(tradesNew[winner] ~= nil and tradesNew[winner].winners[looser] ~= nil) then
		if(tradesNew[winner].winners[looser] - win < 0) then
			if(tradesNew[looser] == nil) then
				tradesNew[looser] = {};
			end
			if(tradesNew[looser].winners == nil) then
				tradesNew[looser].winners = {};
			end
			tradesNew[looser].winners[winner] = win - tradesNew[winner].winners[looser];
			tradesNew[winner].winners[looser] = nil;
		else
			tradesNew[winner].winners[looser] = tradesNew[winner].winners[looser] - win;
		end
	else
		if(tradesNew[looser] == nil) then
			tradesNew[looser] = {};
		end
		if(tradesNew[looser].winners == nil) then
			tradesNew[looser].winners = {};
		end
		tradesNew[looser].winners[winner] = win;
	end
	
	MorthalinRaidTools:ResetGamble();
end

function MorthalinRaidTools:MoreLow()
	local playersString = "";
	countInvitedR = 0;
	countRolledR = 0;
	for name, roll in pairs(low) do
		playersString = playersString .. name .. ", ";
		countInvitedR = countInvitedR + 1;
		if(name == UnitName("player")) then
			btnRoll:SetDisabled(false);
		end
	end
	playersString = string.sub(playersString, 1, -3);
	SendChatMessage("---Players " .. playersString .. " have same roll. ---", "RAID", "Common");
	SendChatMessage("---Write /roll " .. count .. " again. ---", "RAID", "Common");
	rollType = 2;
end

function MorthalinRaidTools:LowRerolled()
	rollType = 0;
	local rollersSorted = {};
	for name, roll in pairs(low) do
		table.insert(rollersSorted , {name = name, roll = roll});
	end
	local function sortRollers(a, b)
		if(a.roll == b.roll) then
			return a.name < b.name;
		else
			return a.roll < b.roll;
		end
	end
	table.sort(rollersSorted, sortRollers);
	
	low = {};
	
	local minimumR = rollersSorted[1].roll;
	local minCount = 0;
	local index = 1;
	local looping = true;
	while looping do
		if(index > #rollersSorted or rollersSorted[index].roll ~= minimumR) then
			looping = false;
		else
			low[rollersSorted[index].name] = 0;
			index = index + 1;
			minCount = minCount + 1;
		end
	end
	
	local maxCount = 0;
	for name, roll in pairs(high) do
		maxCount = maxCount + 1;
	end
	
	if(minCount > 1) then
		MorthalinRaidTools:MoreLow();
	elseif(maxCount > 1) then
		MorthalinRaidTools:MoreHigh();
	else
		MorthalinRaidTools:GetWinnerLooser();
	end
end

function MorthalinRaidTools:MoreHigh()
	local playersString = "";
	countInvitedR = 0;
	countRolledR = 0;
	for name, roll in pairs(high) do
		playersString = playersString .. name .. ", ";
		countInvitedR = countInvitedR + 1;
		if(name == UnitName("player")) then
			btnRoll:SetDisabled(false);
		end
	end
	playersString = string.sub(playersString, 1, -3);
	SendChatMessage("---Players " .. playersString .. " have same roll. ---", "RAID", "Common");
	SendChatMessage("---Write /roll " .. count .. " again. ---", "RAID", "Common");
	rollType = 3;
end

function MorthalinRaidTools:HighRerolled()
	rollType = 0;
	local rollersSorted = {};
	for name, roll in pairs(high) do
		table.insert(rollersSorted , {name = name, roll = roll});
	end
	local function sortRollers(a, b)
		if(a.roll == b.roll) then
			return a.name < b.name;
		else
			return a.roll < b.roll;
		end
	end
	table.sort(rollersSorted, sortRollers);
	
	high = {};
	
	local maximumR = rollersSorted[#rollersSorted].roll;
	local maxCount = 0;
	local index = #rollersSorted;
	local looping = true;
	while looping do
		if(index < 1 or rollersSorted[index].roll ~= maximumR) then
			looping = false;
		else
			high[rollersSorted[index].name] = 0;
			index = index - 1;
			maxCount = maxCount + 1;
		end
	end
	
	if(maxCount > 1) then
		MorthalinRaidTools:MoreHigh();
	else
		MorthalinRaidTools:GetWinnerLooser();
	end
end

function MorthalinRaidTools:GetGlobalStatistics()
	local statsSort = {};
	for name, count in pairs(MorthalinRaidTools.db.global.gambleStatistics) do
		table.insert(statsSort, {name = name, count = count});
	end
	
	local function sortGamblers(a, b)
		if(a.count == b.count) then
			return a.name < b.name;
		else
			return a.count > b.count;
		end
	end
	table.sort(statsSort, sortGamblers);
	if(#statsSort >= 10) then
		SendChatMessage("--- Global Gamble Statistics ---", "RAID", "Common");
		SendChatMessage("--- Top 5 ---", "RAID", "Common");
		for i = 1, 5 do
			SendChatMessage(statsSort[i].name .. ": " .. statsSort[i].count .. "g", "RAID", "Common");
		end
		SendChatMessage("--- Bottom 5 ---", "RAID", "Common");
		for i = 0, 4 do
			SendChatMessage(statsSort[#statsSort - 4 + i].name .. ": " .. statsSort[#statsSort - 4 + i].count .. "g", "RAID", "Common");
		end
	else
		SendChatMessage("--- Global Gamble Statistics ---", "RAID", "Common");
		for i = 1, #statsSort do
			SendChatMessage(statsSort[i].name .. ": " .. statsSort[i].count .. "g", "RAID", "Common");
		end
	end
end

function MorthalinRaidTools:GetLocalStatistics()
	local statsSort = {};
	for name, count in pairs(localGambleStatistics) do
		table.insert(statsSort, {name = name, count = count});
	end
	
	local function sortGamblers(a, b)
		if(a.count == b.count) then
			return a.name < b.name;
		else
			return a.count > b.count;
		end
	end
	table.sort(statsSort, sortGamblers);
	if(#statsSort >= 10) then
		SendChatMessage("--- Actual Gamble Statistics ---", "RAID", "Common");
		SendChatMessage("--- Top 5 ---", "RAID", "Common");
		for i = 1, 5 do
			SendChatMessage(statsSort[i].name .. ": " .. statsSort[i].count .. "g", "RAID", "Common");
		end
		SendChatMessage("--- Bottom 5 ---", "RAID", "Common");
		for i = 0, 4 do
			SendChatMessage(statsSort[#statsSort - 4 + i].name .. ": " .. statsSort[#statsSort - 4 + i].count .. "g", "RAID", "Common");
		end
	else
		SendChatMessage("--- Actual Gamble Statistics ---", "RAID", "Common");
		for i = 1, #statsSort do
			SendChatMessage(statsSort[i].name .. ": " .. statsSort[i].count .. "g", "RAID", "Common");
		end
	end
end

function MorthalinRaidTools:ShowTrades()
	local tradesSorted = {};
	for looser, winners in pairs(tradesNew) do
		table.insert(tradesSorted, {name = looser, winners = {}});
		for winner, count in pairs(tradesNew[looser].winners) do
			table.insert(tradesSorted[#tradesSorted].winners, {name = winner, count = count});
		end
	end
	local function sortTrades(a, b)
		return a.name < b.name;
	end
	table.sort(tradesSorted, sortTrades);
	for i = 1, #tradesSorted do
		table.sort(tradesSorted[i].winners, sortTrades);
	end
	SendChatMessage("--- All trades ---", "RAID", "Common");
	for i = 1, #tradesSorted do
		for j = 1, #tradesSorted[i].winners do
			SendChatMessage(tradesSorted[i].name .. " -> " .. tradesSorted[i].winners[j].name .. ": " .. tostring(tradesSorted[i].winners[j].count) .. "g", "RAID", "Common");
		end
	end
end

function MorthalinRaidTools:CallbackStartRolling(player, roll, range)
	if(rollType == 1) then
		if(range == "(1-" .. count .. ")") then
			if(rollers[player] == 0) then
				rollers[player] = tonumber(roll);
				countRolled = countRolled + 1;
			end
		else
			SendChatMessage(player .. " please write /roll " .. count .. ".", "RAID", "Common");
		end
		
		if(countInvited == countRolled) then
			MorthalinRaidTools:AllRolled();
		end
	elseif(rollType == 2) then
		if(range == "(1-" .. count .. ")") then
			if(low[player] == 0) then
				low[player] = tonumber(roll);
				countRolledR = countRolledR + 1;
			end
		else
			SendChatMessage(player .. " please write /roll " .. count .. ".", "RAID", "Common");
		end
		if(countInvitedR == countRolledR) then
			if(countInvited == countInvitedR) then
				rollers = low;
				MorthalinRaidTools:AllRolled();
			else
				MorthalinRaidTools:LowRerolled();
			end
		end
	elseif(rollType == 3) then
		if(range == "(1-" .. count .. ")") then
			if(high[player] == 0) then
				high[player] = tonumber(roll);
				countRolledR = countRolledR + 1;
			end
		else
			SendChatMessage(player .. " please write /roll " .. count .. ".", "RAID", "Common");
		end
		if(countInvitedR == countRolledR) then
			MorthalinRaidTools:HighRerolled();
		end
	end
end
