local AceGUI = LibStub("AceGUI-3.0")

local maxWidth;
local usedBuffs;
local usedBuffsTypes;

MorthalinRaidTools.spellTypeStrings = {
	[1] = "WB",
	[2] = "Cons",
	[3] = "ProtP",
	[4] = "Class",
	[5] = "Other"
}

function MorthalinRaidTools:ConstructIconTable(object, data, buffTime)
	if(self.ui.iconTable ~= nil) then
		self.ui.iconTable:Hide();
	end
	self.ui.iconTable = nil;
	if(self.ui.iconTable == nil) then
		local FrameBackdrop = {
			bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
			tile = true, tileSize = 32, edgeSize = 16,
			insets = { left = 5, right = 5, top = 5, bottom = 5 }
		}
		
		self.ui.iconTable = CreateFrame("Frame",nil,object,"BackdropTemplate");
		self.ui.iconTable:SetFrameStrata("TOOLTIP");
		self.ui.iconTable:SetWidth(130);
		self.ui.iconTable:SetHeight(100);
		maxWidth = 130;

		self.ui.iconTable:EnableMouse(true);
		self.ui.iconTable:SetMovable(false);
		self.ui.iconTable:SetResizable(false);
		self.ui.iconTable:SetBackdrop(FrameBackdrop);
		self.ui.iconTable:SetBackdropColor(0, 0, 0, 1);
		self.ui.iconTable:SetToplevel(true);
		
	end
	self:SetUsedBuffs(data);
	self:SetIconTableRows(data, buffTime);
	self.ui.iconTable:SetPoint("LEFT", (self.ui.iconTable:GetWidth() * (-1)), 0);
end

function MorthalinRaidTools:SetIconTableIcons(data, top)
	local index = 1;
	local left = 130;
	local buffsSorted = {};
	local activeType = 1;
	local rest = (usedBuffsTypes[1] or 0) * 18;

	if(data.buffs ~= nil) then
		for name, value in pairs(data.buffs) do
			table.insert(buffsSorted,{name = name, data = value});
		end
		local function sortBuffs(a, b)
			if(usedBuffs[a.name:gsub("%s+", "")] == nil and usedBuffs[b.name:gsub("%s+", "")] == nil) then
				return a.name:gsub("%s+", "") < b.name:gsub("%s+", "");
			elseif(usedBuffs[a.name:gsub("%s+", "")] == nil) then
				return false;
			elseif(usedBuffs[b.name:gsub("%s+", "")] == nil) then
				return true;
			else
				if((usedBuffs[a.name:gsub("%s+", "")].spellType == nil and 5 or usedBuffs[a.name:gsub("%s+", "")].spellType) == (usedBuffs[b.name:gsub("%s+", "")].spellType == nil and 5 or usedBuffs[b.name:gsub("%s+", "")].spellType)) then
					return a.name:gsub("%s+", "") < b.name:gsub("%s+", "");
				else
					return (usedBuffs[a.name:gsub("%s+", "")].spellType == nil and 5 or usedBuffs[a.name:gsub("%s+", "")].spellType) < (usedBuffs[b.name:gsub("%s+", "")].spellType == nil and 5 or usedBuffs[b.name:gsub("%s+", "")].spellType);
				end
			end
		end
		table.sort(buffsSorted, sortBuffs);
		for i = 1, #buffsSorted do
			for j = 1, #buffsSorted[i].data do
				if(MorthalinRaidTools.db.global.showBuffsAll or MorthalinRaidTools:CheckSettings(buffsSorted[i], data.class, data.name, j)) then
					if(usedBuffs[buffsSorted[i].name:gsub("%s+", "")] ~= nil) then
						if(usedBuffs[buffsSorted[i].name:gsub("%s+", "")].spellType ~= activeType) then
							left = left + rest + 20;
							if(usedBuffsTypes[1] == nil and activeType == 1) then
								left = left - 20;
							end
							while(usedBuffs[buffsSorted[i].name:gsub("%s+", "")].spellType ~= activeType + 1) do
								if(activeType == 5) then
									break;
								end
								left = left + ((usedBuffsTypes[activeType + 1] or 0) * 18) + ((usedBuffsTypes[activeType + 1] and 1 or 0) * 20);
								activeType = activeType + 1;
							end
							activeType = (usedBuffs[buffsSorted[i].name:gsub("%s+", "")].spellType or 5);
							rest = (usedBuffsTypes[usedBuffs[buffsSorted[i].name:gsub("%s+", "")].spellType] or 0) * 18;
						end
						local iconGUI = CreateFrame("BUTTON", nil, self.ui.iconTable, "SecureHandlerClickTemplate");
						iconGUI:SetSize(17, 17);
						iconGUI:SetPoint("TOPLEFT", left, top * (-1));
						
						local image = iconGUI:CreateTexture(nil,"BACKGROUND",nil,-6);
						image:SetTexture(buffsSorted[i].data[j].icon);
						image:SetTexCoord(0, 1, 0, 1);
						image:SetAllPoints(iconGUI);
						iconGUI.name = buffsSorted[i].name;
						iconGUI:SetScript("OnEnter", function()
							GameTooltip:SetOwner(iconGUI,"ANCHOR_LEFT");
							GameTooltip:AddLine(buffsSorted[i].name);
							GameTooltip:Show();
						end);
						iconGUI:SetScript("OnLeave", function() GameTooltip:Hide(); end);
						iconGUI.text = iconGUI:CreateFontString(nil, "ARTWORK");
						iconGUI.text:SetFont(NumberFontNormalYellow:GetFont());
						iconGUI.text:SetJustifyH("CENTER");
						iconGUI.text:SetJustifyV("CENTER");
						if(buffsSorted[i].data[j].count ~= nil and buffsSorted[i].data[j].count > 1) then
							iconGUI.text:SetText(tostring(buffsSorted[i].data[j].count == nil and 1 or buffsSorted[i].data[j].count));
						end
						iconGUI.text:SetAllPoints(iconGUI);
						left = left + 18;
						rest = rest - 18;
					end
				end
			end
		end
	end
	if(maxWidth < left) then
		maxWidth = left;
		self.ui.iconTable:SetWidth(left + 20);
	end
end

function MorthalinRaidTools:SetIconTableRows(data, buffTime)
	local top = 7;
	local index = 1;
	if(buffTime ~= nil) then
		local row = CreateFrame("Frame", nil, self.ui.iconTable,"BackdropTemplate");
		row:SetWidth(120);
		row:SetHeight(10);
		row:SetPoint("TOPLEFT", 7, top * (-1));
		row.text = row:CreateFontString(nil, "ARTWORK");
		row.text:SetFont(GameFontNormal:GetFont());
		row.text:SetJustifyH("LEFT");
		row.text:SetJustifyV("TOP");
		row.text:SetText(date("%d.%m.%y %H:%M:%S",buffTime));
		row.text:SetAllPoints(row);
		
		local left = 130;
		local sortSpellType = {};
		for spellType, count in pairs(usedBuffsTypes) do
			table.insert(sortSpellType,{spellType = spellType, count = count});
		end
		local function sortType(a, b)
			return a.spellType < b.spellType;
		end
		table.sort(sortSpellType, sortType);
		for i = 1, #sortSpellType do
			row = CreateFrame("Frame", nil, self.ui.iconTable,"BackdropTemplate");
			row:SetWidth(120);
			row:SetHeight(10);
			row:SetPoint("TOPLEFT", left, top * (-1));
			left = left + (sortSpellType[i].count * 18) + 20;
			row.text = row:CreateFontString(nil, "ARTWORK");
			row.text:SetFont(GameFontNormal:GetFont());
			row.text:SetJustifyH("LEFT");
			row.text:SetJustifyV("TOP");
			row.text:SetText(MorthalinRaidTools.spellTypeStrings[sortSpellType[i].spellType]);
			row.text:SetAllPoints(row);
		end
		
		top = top + 18;
	end
	for i = 1, #data do
		if(data[i].group ~= 0) then
			local row = CreateFrame("Frame", nil, self.ui.iconTable,"BackdropTemplate");
			row:SetWidth(100);
			row:SetHeight(10);
			row:SetPoint("TOPLEFT", 7, top * (-1));
			row.text = row:CreateFontString(nil, "ARTWORK");
			row.text:SetFont(GameFontNormal:GetFont());
			row.text:SetJustifyH("LEFT");
			row.text:SetJustifyV("TOP");
			row.text:SetText("|cff" .. data[i].color .. data[i].name);
			row.text:SetAllPoints(row);
			MorthalinRaidTools:SetIconTableIcons(data[i], top);
			top = top + 18;
			index = index + 1;
		end
	end
	self.ui.iconTable:SetHeight(top + 4);
end

function MorthalinRaidTools:CheckSettings(buff, class, name, index)
	local ret = false;
	for i = 1, #self.db.global.buffs do
		if(buff.name:gsub("%s+", "") == self.db.global.buffs[i].name:gsub("%s+", "") and (buff.data[index].spellId == 0 or self.db.global.buffs[i].spellId == nil or buff.data[index].spellId == self.db.global.buffs[i].spellId)) then
			for j = 1, #self.db.global.buffs[i].classes do
				if(class == self.db.global.buffs[i].classes[j]) then
					ret = true;
					j = #self.db.global.buffs[i];
				end
			end
			if(ret == false) then
				for j = 1, #self.db.global.buffs[i].include do
					if(name == self.db.global.buffs[i].include[j]) then
						ret = true;
					end
				end
			end
			if(ret == true) then
				for j = 1, #self.db.global.buffs[i].exclude do
					if(name == self.db.global.buffs[i].exclude[j]) then
						ret = false;
					end
				end
			end
			i = #self.db.global.buffs;
		end
	end
	return ret;
end

function MorthalinRaidTools:SetUsedBuffs(data)
	usedBuffs = {};
	usedBuffsTypes = {};
	for i = 1, #data do
		for name, value in pairs(data[i].buffs) do
			if(usedBuffs[name:gsub("%s+", "")] == nil) then
				if(#value ~= 1) then
					usedBuffs[name:gsub("%s+", "")] = {spellType = 0, spellId = 0};
				else
					usedBuffs[name:gsub("%s+", "")] = {spellType = 0, spellId = value[1].spellId};
				end
			end
		end
	end
	
	for i = 1, #self.db.global.buffs do
		if(usedBuffs[self.db.global.buffs[i].name:gsub("%s+", "")] ~= nil and (usedBuffs[self.db.global.buffs[i].name:gsub("%s+", "")].spellId == 0 or self.db.global.buffs[i].spellId == nil or usedBuffs[self.db.global.buffs[i].name:gsub("%s+", "")].spellId == self.db.global.buffs[i].spellId)) then
			usedBuffs[self.db.global.buffs[i].name:gsub("%s+", "")].spellType = (self.db.global.buffs[i].spellType or 5);
		end
	end
	
	for i = 1, #data do
		local usedBuffsTypesTemp = {};
		for name, value in pairs(data[i].buffs) do
			if(usedBuffsTypesTemp[usedBuffs[name:gsub("%s+", "")].spellType or 5] == nil) then
				usedBuffsTypesTemp[usedBuffs[name:gsub("%s+", "")].spellType or 5] = #value;
			else
				usedBuffsTypesTemp[usedBuffs[name:gsub("%s+", "")].spellType or 5] = usedBuffsTypesTemp[usedBuffs[name:gsub("%s+", "")].spellType  or 5] + #value;
			end
		end
		for spellType, count in pairs(usedBuffsTypesTemp) do
			if((usedBuffsTypesTemp[spellType] or 0) > (usedBuffsTypes[spellType] or 0)) then
				usedBuffsTypes[spellType] = usedBuffsTypesTemp[spellType];
			end
		end
	end
	
	if(MorthalinRaidTools.db.global.showBuffsAll == false) then
		usedBuffsTypes[0] = nil;
		for name, data in pairs(usedBuffs) do
			if(data.spellType == 0) then
				usedBuffs[name] = nil;
			end
		end
	else
		usedBuffsTypes[5] = (usedBuffsTypes[5] or 0) + (usedBuffsTypes[0] or 0);
		for name, data in pairs(usedBuffs) do
			if(data.spellType == 0) then
				usedBuffs[name].spellType = 5;
			end
		end
		usedBuffsTypes[0] = nil;
		if(usedBuffsTypes[5] == 0) then
			usedBuffsTypes[5] = nil;
		end
	end
end
